# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os
import json

from django.utils.translation import ugettext as _
from django import forms
from django.http import HttpResponse
from django.forms.formsets import formset_factory
from project.view import RootView
from project.view import FuncView as _View

from project.fal.cloud import (
    obtain_profiles,
    create_profile,
    update_profile,
    delete_profile,
    check_profile,
)

from . import fields


def _get_profiles():
    return [{
        'pk': profile['id'],
        'name': profile['name'],
        'contractor': profile['contractor'],
        'plan': profile['plan'],
        'authmethod': profile['authmethod'],
        'otherauthmethod': profile['otherauthmethod'],
        'terminalkey': profile['terminalkey'],
        'clientid': profile['clientid'],
        'password': profile['password'],
    } for profile in obtain_profiles('fletsazukeru')]


def _get_initials(profiles):
    return [{ 'pk': profile['pk'] } for profile in profiles]


class CreateForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'create')
        super(CreateForm, self).__init__(**kwargs)

    name = fields.NameField()
    contractor = fields.ContractorField()
    plan = fields.PlanField()
    authmethod = fields.AuthMethodField()
    otherauthmethod = fields.OtherAuthMethodField()
    terminalkey = fields.TerminalKeyField()
    clientid = fields.ClientIdField()
    password = fields.PasswordField()


class UpdateForm(CreateForm):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'update')
        super(UpdateForm, self).__init__(**kwargs)

    pk = forms.CharField(widget=forms.widgets.HiddenInput)


class DeleteForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'delete')
        super(DeleteForm, self).__init__(**kwargs)

    pk = forms.CharField(widget=forms.widgets.HiddenInput)


class ProfilesView(_View):

    views = ['create', 'update', 'delete']

    def view(self, request):
        # profiles = _get_profiles()
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm()

        template = os.path.join(self.TEMPLATES, 'profiles.html')
        dictionary = {
            'profiles': _get_profiles(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_create(self, request):
        # profiles = _get_profiles()
        create_form = CreateForm(data=request.POST)
        update_form = UpdateForm()
        delete_form = DeleteForm()
        form = create_form

        with self.processing(constname='ERR_CLOUD_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                name = form.cleaned_data['name']
                contractor = form.cleaned_data['contractor']
                plan = form.cleaned_data['plan']
                authmethod = form.cleaned_data['authmethod']
                otherauthmethod = form.cleaned_data['otherauthmethod']
                terminalkey = form.cleaned_data['terminalkey']
                clientid = form.cleaned_data['clientid']
                password = form.cleaned_data['password']
                ret = create_profile('fletsazukeru', {
                    'name': name,
                    'contractor': contractor,
                    'plan': plan,
                    'authmethod': authmethod,
                    'otherauthmethod': otherauthmethod,
                    'terminalkey': terminalkey,
                    'clientid': clientid,
                    'password': password,
                })
                is_succeeded = ret['is_succeeded']
                message = ret['message']
                proc.result(is_succeeded)
                self.set_message(message)
                if is_succeeded:
                    form = create_form = CreateForm()

        template = os.path.join(self.TEMPLATES, 'profiles.html')
        dictionary = {
            'profiles': _get_profiles(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_update(self, request):
        # profiles = _get_profiles()
        create_form = CreateForm()
        update_form = UpdateForm(data=request.POST)
        delete_form = DeleteForm()
        form = update_form

        with self.processing(constname='ERR_CLOUD_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                pk = form.cleaned_data['pk']
                name = form.cleaned_data['name']
                contractor = form.cleaned_data['contractor']
                plan = form.cleaned_data['plan']
                authmethod = form.cleaned_data['authmethod']
                otherauthmethod = form.cleaned_data['otherauthmethod']
                terminalkey = form.cleaned_data['terminalkey']
                clientid = form.cleaned_data['clientid']
                password = form.cleaned_data['password']
                ret = update_profile('fletsazukeru', {
                    'id': pk,
                    'name': name,
                    'contractor': contractor,
                    'plan': plan,
                    'authmethod': authmethod,
                    'otherauthmethod': otherauthmethod,
                    'terminalkey': terminalkey,
                    'clientid': clientid,
                    'password': password,
                })
                is_succeeded = ret['is_succeeded']
                message = ret['message']
                proc.result(is_succeeded)
                self.set_message(message)
                if is_succeeded:
                    form = update_form = UpdateForm()

        template = os.path.join(self.TEMPLATES, 'profiles.html')
        dictionary = {
            'profiles': _get_profiles(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_delete(self, request):
        # profiles = _get_profiles()
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm(data=request.POST)
        form = delete_form

        with self.processing(constname='ERR_CLOUD_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                pk = form.cleaned_data['pk']
                ret = delete_profile('fletsazukeru', pk)
                is_succeeded = ret['is_succeeded']
                message = ret['message']
                proc.result(is_succeeded)
                self.set_message(message)
                if is_succeeded:
                    form = delete_form = DeleteForm()

        template = os.path.join(self.TEMPLATES, 'profiles.html')
        dictionary = {
            'profiles': _get_profiles(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


class TestView(RootView):
    def view(self, request, pk=None):
        success = False
        try:
            success = check_profile('fletsazukeru', pk)
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                json.dumps({
                    "success": success,
                }),
                mimetype='application/json;charset=UTF-8;',
            ),
        }
