#!/usr/bin/env python
# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

from django import forms
from project.fal.cloud import obtain_profiles as obtain_connections

from .. import base


class _ConnectionField(forms.ChoiceField):

    def __init__(self, **kwargs):
        kwargs['label'] = kwargs.get('label', '接続情報')
        kwargs['choices'] = kwargs.get('choices', self._get_default_choices())
        super(_ConnectionField, self).__init__(**kwargs)

    def _get_default_choices(self):
        choices = [('', '選択してください')]
        for connection in obtain_connections('fletsazukeru'):
            choices.append((connection['id'], connection['name']))
        return choices


class _FolderField(forms.ChoiceField):
    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', 'フォルダー')
        kwargs['widget'] = kwargs.get('widget', forms.widgets.RadioSelect)
        kwargs['choices'] = kwargs.get('choices', [
            ('my', 'マイフォルダー'),
            ('group', 'グループフォルダー'),
        ])
        kwargs['initial'] = kwargs.get('initial', kwargs['choices'][0][0])
        super(_FolderField, self).__init__(*args, **kwargs)


class _SubdirField(forms.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', 'サブフォルダー')
        super(_SubdirField, self).__init__(*args, **kwargs)


class Form(base.BaseForm):

    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(*args, **kwargs)
        self.fields['connection_id'] = _ConnectionField()
        self.fields['folder'] = _FolderField()
        self.fields['subdir'] = _SubdirField(required=False)
